use bevy::prelude::*;
use std::time::Duration;

pub mod components;
pub mod constants;
pub mod resources;
pub mod game_state;
pub mod player;
pub mod enemy;
pub mod bullet;
pub mod stage;
pub mod systems;

use constants::{PLAYER_RESPAWN_DELAY, STARTING_LIVES, WINDOW_HEIGHT, WINDOW_WIDTH};
use resources::{
    AttackDiveTimer, CurrentStage, EnemySpawnTimer, FormationState, PlayerLives,
    PlayerRespawnTimer, Score,
};
use game_state::{
    cleanup_game_entities, cleanup_game_over_ui, setup_game_over_ui, AppState,
};
use player::{
    check_player_enemy_collisions, manage_invincibility, move_player, player_shoot,
    respawn_player,
};
use enemy::{
    check_formation_complete, enemy_shoot, is_formation_complete, move_enemies, spawn_enemies,
    trigger_attack_dives,
};
use bullet::{
    check_bullet_collisions, check_enemy_bullet_player_collisions, move_bullets,
    move_enemy_bullets,
};
use stage::check_stage_clear;
use systems::{player_exists, player_vulnerable, setup, should_respawn_player, update_window_title};

fn main() {
    App::new()
        .init_state::<AppState>() // Initialize the AppState
        .insert_resource(ClearColor(Color::rgb(0.0, 0.0, 0.0)))
        .add_plugins(DefaultPlugins.set(WindowPlugin {
            primary_window: Some(Window {
                title: "Galaga :: Stage: 1 Lives: 3 Score: 0".into(), // Initial title
                resolution: (WINDOW_WIDTH, WINDOW_HEIGHT).into(),
                ..default()
            }),
            ..default()
        }))
        // Add Resources
        .insert_resource(PlayerLives {
            count: STARTING_LIVES,
        })
        .insert_resource(PlayerRespawnTimer {
            timer: Timer::new(
                Duration::from_secs_f32(PLAYER_RESPAWN_DELAY),
                TimerMode::Once,
            ),
        })
        .insert_resource(EnemySpawnTimer {
            timer: Timer::new(Duration::from_secs_f32(0.5), TimerMode::Repeating),
        })
        .insert_resource(Score { value: 0 })
        .insert_resource(CurrentStage {
            number: 1,
            waiting_for_clear: false,
        })
        .insert_resource(FormationState {
            next_slot_index: 0,
            total_spawned_this_stage: 0,
            formation_complete: false,
        })
        .insert_resource(AttackDiveTimer {
            timer: {
                let mut timer = Timer::new(Duration::from_secs_f32(3.0), TimerMode::Repeating);
                timer.pause(); // Start paused
                timer
            },
        })
        // Add Systems
        .add_systems(Startup, setup)
        // Systems running only when Playing
        .add_systems(
            Update,
            (
                // Player systems
                move_player,
                player_shoot.run_if(player_exists),
                check_player_enemy_collisions.run_if(player_vulnerable),
                respawn_player.run_if(should_respawn_player),
                manage_invincibility,
                // Bullet systems
                move_bullets,
                check_bullet_collisions,
                move_enemy_bullets,
                check_enemy_bullet_player_collisions.run_if(player_vulnerable),
                // Enemy systems
                spawn_enemies,
                move_enemies,
                enemy_shoot, // Consider run_if attacking state? (Handled internally for now)
            )
                .run_if(in_state(AppState::Playing)),
        )
        // Formation/Attack/Stage systems (some need specific ordering or run conditions)
        .add_systems(
            Update,
            (
                check_formation_complete,
                trigger_attack_dives.run_if(is_formation_complete), // Run only when formation is complete
                check_stage_clear, // Uses world access, run separately
            )
                .chain() // Ensure these run in order if needed, check_formation first
                .run_if(in_state(AppState::Playing)),
        )
        // Systems running regardless of state (or managing state transitions)
        .add_systems(
            Update,
            (
                update_window_title, // Keep title updated
                // TODO: Add system to check for restart input in GameOver state
                bevy::window::close_on_esc, // Allow closing anytime
            ),
        )
        // Systems for entering/exiting states
        .add_systems(OnEnter(AppState::GameOver), setup_game_over_ui)
        .add_systems(OnExit(AppState::GameOver), cleanup_game_over_ui)
        .add_systems(OnExit(AppState::Playing), cleanup_game_entities) // Cleanup when leaving Playing
        .run();
}
